"""Creates a new enviroment stage."""
from django.core.management.base import BaseCommand

from ...crypto import StateList


class Command(BaseCommand):
    help = 'Add a enviroment variable using a key'

    def add_arguments(self, parser):
        parser.add_argument('-k', '--key', type=str)
        parser.add_argument('-n', '--name', type=str)

    def handle(self, *args, name=None, key=None, ** options):
        """Create a new enviroment file with the name and a new KEY."""
        stl = StateList(key=key)

        # we are going to either show a variable or all of the variables from the enviroment
        print("Active enviroment:", stl.name)
        list_of_names = [name]
        if name is None:
            list_of_names = stl.get_names_from_active()

        # we show all
        for name in list_of_names:
            print(name, stl.get_from_active(name))



"""Creates a new enviroment stage."""
from django.core.management.base import BaseCommand

from ...crypto import StateList


class Command(BaseCommand):
    help = 'Add a enviroment variable using a key'

    def add_arguments(self, parser):
        parser.add_argument('name', type=str)
        parser.add_argument('value', type=str)
        parser.add_argument('-k', '--key', type=str)

    def handle(self, *args, name=None,  value=None, key=None, ** options):
        """Create a new enviroment file with the name and a new KEY."""
        stl = StateList(key=key)
        print("Adding to variable to enviroment", stl.name)
        stl.add_active(name.upper(), value)
        stl.save_active()



"""LevelConfig to describe levels."""
import os
import sys
from enum import Enum

from .crypto import NoEnvKeyFound, StateList


# from socket import gethostbyname, gethostname


class Deployment(Enum):
    """A basic run level based on the enviroment variables

    You can create your own deployment by creating a Enum with multiple deployment types
    """

    DEBUG = 'debug'
    STAGING = 'staging'
    PRODUCTION = 'production'


class NoEnvFileFound(Exception):
    """Warns the user that we haven't defined any enviroment class."""

    pass


class DeployLevel(object):
    """Configuration for the several run levels."""

    def __init__(self, levels=None):
        """Set the level using the enviroment variable."""
        if levels is None:
            levels = Deployment
        assert not isinstance(
            levels, Enum), "Pleace pass a Enum as the run levels"

        self.parent = sys.modules[os.environ.get("DJANGO_SETTINGS_MODULE")]
        try:
            self.stl = StateList()
        except NoEnvKeyFound as err:
            self.stl = None
            print("Warning - You haven't setup your enviroment KEY.")

        self.levels = levels
        self.current_level = None

        # figure out the run level, using the key
        if self.stl is not None:
            self.find_current_level()
            self.load_globals()

    def find_current_level(self):
        """Find the current level."""

        for level in self.levels:
            if level.value == self.stl.name:
                self.current_level = level

        if self.current_level is None:
            # we could not find a env level for this level
            raise NoEnvFileFound(
                "Could not find encrypted env for the level {}".format(self.stl.name))

    def load_globals(self):
        """Load all enviroment variables into globals."""
        for name in self.stl.get_names_from_active():
            setattr(self.parent, name, self.stl.get_from_active(name))

    @property
    def LEVEL(self):
        return self.current_level

    def __str__(self):
        return "Deployment: {}".format(self.current_level)




"""Creates a new enviroment stage."""
import json
import random

from django.core.management.base import BaseCommand

from ...crypto import Encrypter


class Command(BaseCommand):
    help = 'Create a new enviroment, with a .env file and a new KEY'

    def add_arguments(self, parser):
        parser.add_argument('enviroment_name', type=str)
        parser.add_argument('--parent', type=str)

    def handle(self, *args, enviroment_name=None, parent=False, **options):
        """Create a new enviroment file with the name and a new KEY."""
        print("Creating a new enviroment file", enviroment_name)
        new_key = Encrypter.generate_key()
        encrypter = Encrypter(new_key)
        result = {}
        result['name'] = enviroment_name
        result['signed_name'] = encrypter.encrypt(enviroment_name)

        # set the django secret key
        secret_key = ''.join(random.SystemRandom().choice(
            'abcdefghijklmnopqrstuvwxyz0123456789!@#$%^&*(-_=+)') for i in range(50))
        result['SECRET_KEY'] = encrypter.encrypt(secret_key)

        if parent:
            result['parent'] = parent

        with open('{}.env'.format(enviroment_name), 'w') as f:
            f.write(json.dumps(result, indent=4, sort_keys=True))

        print()
        print("WARNING - Make sure you save the following key, as you will not be able to recover it.")
        print(enviroment_name, '=', new_key.decode())
        print()
        print("You can add our key to your local enviroment using:")
        print("export KEY='{}'".format(new_key.decode()))



"""LevelConfig to describe levels."""
import glob
import json
import os
from binascii import a2b_base64, b2a_base64
from enum import Enum

from cryptography.fernet import Fernet


# from socket import gethostbyname, gethostname


class NoEnvKeyFound(Exception):
    """An error in configuration."""
    pass


class NoEnvFileFound(Exception):
    pass


class NoVarFound(Exception):
    pass


class Encrypter(object):

    @classmethod
    def generate_key(cls):
        """Generate a random key."""
        return Fernet.generate_key()

    def __init__(self, key=None):
        self.fernet = Fernet(key)

    def encrypt(self, message):
        ciphertext = self.fernet.encrypt(message.encode("utf-8"))
        return b2a_base64(ciphertext).decode("utf-8").strip('\n')

    def decrypt(self, digest):
        ciphertext = a2b_base64(digest.encode("utf-8"))
        return self.fernet.decrypt(ciphertext).decode("utf-8")


class StateList(object):
    """Read a list of states."""

    def __init__(self, *args, key=None, **kwargs):
        """Read the list of states."""
        self.key = key
        if key is None:
            # We try to load the key from the enviroment
            self.key = self.read_env("KEY")

        self.encrypter = Encrypter(key=self.key)
        self.states = []
        self.active_state = None

        self.read_list()

    def read_env(self, name):
        """Read a variable name from the enviroment."""
        result = os.environ.get(name, None)
        if result is None:
            raise NoEnvKeyFound(
                "The {} variable is not setup in the enviroment".format(name))
        return result

    def read_list(self):
        """Read the list of files."""
        env_files = glob.glob('*.env')
        for i in range(len(env_files)):
            env_file = env_files[i]
            if self.process_env_file(env_file):
                self.active_state = i

    def process_env_file(self, filepath=None):
        """Process the filepath."""
        env_object = json.loads(open(filepath).read())
        self.states.append(env_object)

        # try to check if we can read this enviroment
        try:
            result = self.encrypter.decrypt(env_object['signed_name'])
        except:
            return False

        return result == env_object['name']

    def add_active(self, name, value):
        """Add the name value to the state."""
        self.states[self.active_state][name] = self.encrypter.encrypt(value)

    def remove_active(self, name):
        """Add the name value to the state."""
        del self.states[self.active_state][name]

    def save_active(self):
        """Save the active state."""
        with open('{}.env'.format(self.states[self.active_state]['name']), 'w') as f:
            f.write(json.dumps(
                self.states[self.active_state], indent=4, sort_keys=True))

    def get_names_from_active(self):
        """Return the names from the active."""
        result = list(self.states[self.active_state].keys())
        result.remove('name')
        result.remove('signed_name')

        return result

    def get_from_active(self, name):
        """Return a value from the active state."""
        if name not in self.states[self.active_state]:
            raise NoVarFound(
                "Variable {} not defined on enviroment {}.".format(name, self.name))

        return self.encrypter.decrypt(self.states[self.active_state][name])

    @property
    def name(self):
        """Return the current state."""
        return self.states[self.active_state]['name']



"""Setup script for creating a pip package."""
from setuptools import find_packages, setup

setup(name='django-envcrypto',
      version='0.8.0.1',
      description='A safe way to store Django Enviromental Variables',
      long_description='Store Django Enviromental Variables for multiple deployments, easy and securely.',
      author='Rogerio Candeias',
      author_email='rogerio.candeias@gmail.com',
      url='https://github.com/FoundersFactory/django-envcrypto',
      license='BSD2',
      packages=find_packages(exclude=['contrib', 'docs', 'tests*']),
      classifiers=[
          'Intended Audience :: Developers',
          'Natural Language :: English',
          'Operating System :: OS Independent',
          'Programming Language :: Python',
          'Programming Language :: Python :: 3.4',
          'Topic :: Software Development :: Libraries :: Python Modules'
      ],
      tests_require=['mock', 'nose', 'coverage', 'urllib3[secure]'],
      install_requires=['cryptography>=2.1.4']
      )
